@@ -1,4 +1,3 @@
-
 # Code flow ---------------------------------------------------------------
 
 
@@ -26,36 +25,23 @@
 #' @rdname auth
 #' @export
 #' @examples
-#'
 #' \dontrun{
-#'   auth_code()
-#'   auth_client()
-#'   auth_user()
-#'   auth_binding()
+#' auth_code()
+#' auth_client()
+#' auth_user()
+#' auth_key()
+#' auth_binding()
 #' }
 #' @returns an `httr2_token`
 auth_code <- function(
     client = Sys.getenv("ARCGIS_CLIENT"),
-    host = arc_host()
-) {
+    host = arc_host()) {
+  check_string(client, allow_empty = FALSE)
+  check_string(host, allow_empty = FALSE)
 
-  token_url <- paste(
-    host,
-    "sharing",
-    "rest",
-    "oauth2",
-    "token",
-    sep = "/"
-  )
+  token_url <- paste(host, "sharing", "rest", "oauth2", "token", sep = "/")
 
-  auth_url <- paste(
-    host,
-    "sharing",
-    "rest",
-    "oauth2",
-    "authorize",
-    sep = "/"
-  )
+  auth_url <- paste(host, "sharing", "rest", "oauth2", "authorize", sep = "/")
 
 
   client <- httr2::oauth_client(
@@ -89,18 +75,22 @@ auth_code <- function(
   )
 
   # add host to the token
-  res[["arcgis_host"]]  <- host
+  res[["arcgis_host"]] <- host
   res
-
-
 }
 # Client auth -------------------------------------------------------------
 #' @export
 #' @rdname auth
-auth_client <- function(client = Sys.getenv("ARCGIS_CLIENT"),
-                        secret = Sys.getenv("ARCGIS_SECRET"),
-                        host = arc_host(),
-                        expiration = 120) {
+auth_client <- function(
+    client = Sys.getenv("ARCGIS_CLIENT"),
+    secret = Sys.getenv("ARCGIS_SECRET"),
+    host = arc_host(),
+    expiration = 120) {
+  check_string(client, allow_empty = FALSE)
+  check_string(secret, allow_empty = FALSE)
+  check_string(host, allow_empty = FALSE)
+  check_number_whole(expiration, min = 5, max = 20160)
+
   # https://developers.arcgis.com/documentation/mapping-apis-and-services/security/application-credentials/
   token_url <- paste(
     host,
@@ -124,7 +114,7 @@ auth_client <- function(client = Sys.getenv("ARCGIS_CLIENT"),
   )
 
   # add host to the token
-  res[["arcgis_host"]]  <- host
+  res[["arcgis_host"]] <- host
   res
 }
 
@@ -161,8 +151,11 @@ auth_user <- function(
     username = Sys.getenv("ARCGIS_USER"),
     password = Sys.getenv("ARCGIS_PASSWORD"),
     host = arc_host(),
-    expiration = 60
-) {
+    expiration = 60) {
+  check_string(username, allow_empty = FALSE)
+  check_string(password, allow_empty = FALSE)
+  check_string(host, allow_empty = FALSE)
+  check_number_whole(expiration, min = 5, max = 20160)
 
   if (expiration > 21600) {
     cli::cli_abort("{.arg expiration} cannot be more than 15 days (21600)")
@@ -203,11 +196,29 @@ auth_user <- function(
     username = username,
     arcgis_host = host,
   )
-
 }
 
 
 
+# API Key ----------------------------------------------------------------
+
+#' @param api_key default `Sys.getenv("ARCGIS_API_KEY")`. A character scalar of an ArcGIS Developer API key.
+#' @export
+#' @rdname auth
+auth_key <- function(api_key = Sys.getenv("ARCGIS_API_KEY"), host = arc_host()) {
+  check_string(
+    api_key,
+    allow_empty = FALSE,
+    allow_na = FALSE,
+    allow_null = FALSE
+  )
+
+  httr2::oauth_token(
+    api_key,
+    arcgis_host = host
+  )
+}
+
 # refreshment mmm tasty --------------------------------------------------------
 
 #' @param token an `httr2_token` as created by `auth_code()` or similar
@@ -216,11 +227,10 @@ auth_user <- function(
 refresh_token <- function(
     token,
     client = Sys.getenv("ARCGIS_CLIENT"),
-    host = arc_host()
-) {
-
+    host = arc_host()) {
   # validate the object is a token
   obj_check_token(token)
+  check_string(client, allow_empty = FALSE)
 
   # extract host from token
   host <- token[["arcgis_host"]]
@@ -243,10 +253,10 @@ refresh_token <- function(
   if (is.null(token[["refresh_token"]])) {
     cli::cli_abort("{.arg token} has expired and no {.field refresh_token} available")
   } else
-    # if it has a refresh check to see if refresh hasn't expired
-    if ((cur_time + token[["refresh_token_expires_in"]]) < cur_time) {
-      cli::cli_abort("Token's {.field refresh_token} has expired.")
-    }
+  # if it has a refresh check to see if refresh hasn't expired
+  if ((cur_time + token[["refresh_token_expires_in"]]) < cur_time) {
+    cli::cli_abort("Token's {.field refresh_token} has expired.")
+  }
 
   # should be able to refresh, go ahead.
   res <- httr2::oauth_flow_refresh(cln, token[["refresh_token"]])
@@ -256,7 +266,6 @@ refresh_token <- function(
   # a refresh token
   res[["arcgis_host"]] <- host
   res
-
 }
 
 #' @rdname auth
@@ -270,11 +279,12 @@ validate_or_refresh_token <- function(
     client = Sys.getenv("ARCGIS_CLIENT"),
     host = arc_host(),
     refresh_threshold = 0,
-    call = rlang::caller_env()
-) {
-
+    call = rlang::caller_env()) {
   # validate the object is a token
   obj_check_token(token, call = call)
+  check_string(client, allow_empty = FALSE)
+  check_string(host, allow_empty = FALSE)
+  check_number_whole(refresh_threshold, min = 0, max = 3600)
 
   cur_time <- as.numeric(Sys.time())
   # check if token is expired or expires within threshold
@@ -286,7 +296,6 @@ validate_or_refresh_token <- function(
     # if not return token
     token
   }
-
 }
 
 #' Determines Portal Host
@@ -303,12 +312,10 @@ validate_or_refresh_token <- function(
 #' A scalar character, `"https://www.arcgis.com"` by default.
 arc_host <- function() {
   host <- Sys.getenv("ARCGIS_HOST")
-
+  check_string(host, allow_empty = TRUE)
   if (host == "") {
     host <- "https://www.arcgis.com"
   }
 
   host
 }
-
-
